﻿FrostyDimUi = AceLibrary("AceAddon-2.0"):new("AceConsole-2.0", "AceEvent-2.0", "AceDB-2.0");

function FrostyDimUi:OnInitialize()
	self.AceOptions = 
	{
		type = 'group',
		args = 
		{
			hotkeys = 
			{
				type = 'toggle',
				name = 'Show HotKeys',
				desc = 'toggle the display of hotkey text on buttons',
				get  = function()
					return self.db.profile.showHotKeys;
				end,
				set	= function(v)
					self.db.profile.showHotKeys = v;
					self:ShowHotKeys(v);
				end,
			},
			macrotitles = 
			{
				type = 'toggle',
				name = 'Show Macro Titles',
				desc = 'toggle the display of macro titles on buttons',
				get  = function()
					return self.db.profile.showMacroTitles;
				end,
				set	= function(v)
					self.db.profile.showMacroTitles = v;
					self:ShowMacroTitles(v);
				end,
			},
			buttonbgs = 
			{
				type = 'toggle',
				name = 'Show Button Backgrounds',
				desc = 'toggle the display of button borders',
				get  = function()
					return self.db.profile.showButtonBackgrounds;
				end,
				set	= function(v)
					self.db.profile.showButtonBackgrounds = v;
					self:ShowButtonBackgrounds(v);
				end,
			},
			plainbuttons = 
			{
				type = 'toggle',
				name = 'Plain Buttons',
				desc = 'toggle the plain buttons',
				get  = function()
					return self.db.profile.showPlainButtons;
				end,
				set	= function(v)
					self.db.profile.showPlainButtons = v;
					self:UsePlainButtons(v);
				end,
			},
		},
	};

	self.DebugLevel = 1; -- 0 = silent, 1 = errors, 2 = info, 3 = debug

	self:RegisterChatCommand( { "/dimui", "/fdu", "/FrostyDimUi" }, self.AceOptions );
	self:RegisterDB("FrostyDimUiDB", "FrostyDimUiDBPC");
	self:RegisterDefaults("profile",
	{
		showHotKeys = true,
		showMacroTitles = true,
		showButtonBackgrounds = true,
		showPlainButtons = false,
	} );
end

function FrostyDimUi:OnEnable()
	self.DimmableButtons = {};
	self:InitializeBlizzardButtons();

	self:ShowHotKeys(self.db.profile.showHotKeys);
	self:ShowMacroTitles(self.db.profile.showMacroTitles);
	self:ShowButtonBackgrounds(self.db.profile.showButtonBackgrounds);
	self:UsePlainButtons(self.db.profile.showPlainButtons);
end

function FrostyDimUi:OnDisable()
	self:ShowHotKeys(true);
	self:ShowMacroTitles(true);
	self:ShowButtonBackgrounds(true);
	self:UsePlainButtons(false);

	self.DimmableButtons = {};
end

function FrostyDimUi:Error( msg )
	if( self.DebugLevel >= 1 ) then
		self:Print( msg );
	end
end

function FrostyDimUi:Info( msg )
	if( self.DebugLevel >= 2 ) then
		self:Print( msg );
	end
end

function FrostyDimUi:Debug( msg )
	if( self.DebugLevel >= 3 ) then
		self:Print( msg );
	end
end


function FrostyDimUi:InitializeBlizzardButtons()
	self:AddDimmableButtonBar( "ActionButton%s", 0, 12 );				-- add the default dimmable buttons

	-- pet bars are not currently working:
	-- Attempt to find 'this' in non-framescript object
	--self:AddDimmableButtonBar( "PetActionButton%s", 0, 10 );			-- pet action bar

	-- added the last two bonus action buttons - thanks to Mikma for the fix
	self:AddDimmableButtonBar( "BonusActionButton%s", 0, 12 );			-- BonusActionButton - what are these?
	self:AddDimmableButtonBar( "ShapeshiftButton%s", 0, 10 );			-- ShapeshiftButton - shape shift buttons, stance buttons, stealth
	self:AddDimmableButtonBar( "MultiBarBottomLeftButton%s", 0, 12 );	-- MultiBarBottomLeft
	self:AddDimmableButtonBar( "MultiBarBottomRightButton%s", 0, 12 );	-- MultiBarBottomRight	
	self:AddDimmableButtonBar( "MultiBarRightButton%s", 0, 12 );		-- MultiBarRight
	self:AddDimmableButtonBar( "MultiBarLeftButton%s", 0, 12 );			-- MultiBarLeft

	self:AddDimmableButtonBar( "CharacterBag%dSlot", -1, 4 );
	self:AddDimmableButton( "MainMenuBarBackpackButton" );
end

function FrostyDimUi:AddDimmableButtonBar( buttonNameFormat, buttonBarIndexOffset, buttonBarCount )
	local a = 1 + buttonBarIndexOffset;
	local b = buttonBarIndexOffset + buttonBarCount;
	local firstButtonIndex = a;

	for i=a, b do
		local buttonName = string.format( buttonNameFormat, i );
		self:AddDimmableButton( buttonName );
	end
end

function FrostyDimUi:AddDimmableButton( buttonName )
	local button = getglobal( buttonName );
	if( button ) then

		-- insert it into the list of dimmable buttons
		table.insert( self.DimmableButtons, button );
	else
		self:Error( "Tried to add invalid button: " .. buttonName );
	end
end

function FrostyDimUi:ShowHotKeys( state )
	for _,button in pairs( self.DimmableButtons ) do
		local hotkey = getglobal( button:GetName() .. "HotKey" );

		if( hotkey ) then
			if( state == false ) then 
				hotkey:Hide();
			else
				hotkey:Show();
			end
		else
			self:Debug( format( "can't find hotkey text for %s", button:GetName() ) );
		end
	end	
end

function FrostyDimUi:ShowMacroTitles( state )
	for _,button in pairs( self.DimmableButtons ) do
		local name = getglobal( button:GetName() .. "Name" );

		if( name ) then
			if( state == false ) then 
				name:Hide();
			else
				name:Show();
			end
		else
			self:Debug( format( "can't find button name for %s", button:GetName() ) );
		end
	end	
end

function FrostyDimUi:ShowButtonBackgrounds( state )

	for _,button in pairs( self.DimmableButtons ) do
		if( state == false ) then
			self:HideButtonBackground( button );
		else	
			self:ShowButtonBackground( button );
		end
	end	
end

function FrostyDimUi:UsePlainButtons( state )
	for _,button in pairs( self.DimmableButtons ) do
		if( state == true ) then
			self:ZoomButtonTexture( button );
		else	
			self:UnZoomButtonTexture( button );
		end
	end	
end

function FrostyDimUi:GetNormalTexture( button )
	if( button ) then
		local buttonBackground = getglobal( button:GetName() .. "NormalTexture" );
		if( buttonBackground == nil ) then
			buttonBackground = getglobal( button:GetName() .. "NormalTexture2" );
		end
		return buttonBackground;
	end
end

function FrostyDimUi:HideButtonBackground( button )
	if( button ) then
		local buttonBackground = self:GetNormalTexture( button );
		if( buttonBackground and buttonBackground.Hide) then
			buttonBackground:Hide();
		else
			self:Debug( "HideButtonBackground failed for " .. button:GetName() );
		end
	end
end

function FrostyDimUi:ShowButtonBackground( button )
	if( button ) then
		local buttonBackground = self:GetNormalTexture( button );
		if( buttonBackground and buttonBackground.Show) then
			buttonBackground:Show();
		else
			self:Debug( "ShowButtonBackground failed for " .. button:GetName() );
		end
	end
end

function FrostyDimUi:GetIconTexture( button )
	if( button ) then
		local icon = getglobal( button:GetName() .. "Icon" );
		if( icon == nil ) then
			-- try "IconTexture"
			icon = getglobal( button:GetName() .. "IconTexture" );
		end
		return icon;
	end
end

function FrostyDimUi:ZoomButtonTexture( button )
	if( button ) then
		local icon = self:GetIconTexture( button );
		if( icon ) then
			icon:SetTexCoord( 0.07, 0.93, 0.07, 0.93 );
		else
			self:Debug( "ZoomButtonTexture failed for " .. button:GetName() );
		end
	end
end

function FrostyDimUi:UnZoomButtonTexture( button )
	if( button ) then
		local icon = self:GetIconTexture( button );
		if( icon ) then
			icon:SetTexCoord(0, 1, 0, 1);
		else
			self:Debug( "UnZoomButtonTexture failed for " .. button:GetName() );
		end
	end	
end